searchNodes=[{"ref":"TwoFactorInACan.html","title":"TwoFactorInACan","module":"TwoFactorInACan","type":"module","doc":"TwoFactorInACan provides easy to use functions for implementing two factor authentication in a phoenix application."},{"ref":"TwoFactorInACan.Hotp.html","title":"TwoFactorInACan.Hotp","module":"TwoFactorInACan.Hotp","type":"module","doc":"Functions for working with the HMAC-based One Time Password algorithm as defined in RFC 4226. For details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt."},{"ref":"TwoFactorInACan.Hotp.html#generate_token/3","title":"TwoFactorInACan.Hotp.generate_token/3","module":"TwoFactorInACan.Hotp","type":"function","doc":"Generates a token from a shared secret and a counter which can be synchronized. This token can be used by one party to verify whether another party has the same secret. Options: :secret_format - the format that the secret is passed in as. Options include: :binary (default) :base32 :base64 :token_length (Default: 6) - the length of the generated token. A longer token is harder to guess and thus more secure. A longer token can also be more difficult for users to accurately transmit. Although everything in TwoFactorInACan supports variable token length, you should be sure that other apps and programs used support the token length set here. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 0) &quot;866564&quot; iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 1) &quot;532769&quot; iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 0, token_length: 10) &quot;1807866564&quot;"},{"ref":"TwoFactorInACan.Secrets.html","title":"TwoFactorInACan.Secrets","module":"TwoFactorInACan.Secrets","type":"module","doc":"Functions for generating cryptographic secrets."},{"ref":"TwoFactorInACan.Secrets.html#generate_totp_secret/1","title":"TwoFactorInACan.Secrets.generate_totp_secret/1","module":"TwoFactorInACan.Secrets","type":"function","doc":"Generates a secret suitable for use in time based one time password (TOTP) two factor authentication. Generates a 160-bit key which is the size recommended by RFC4226 (https://www.ietf.org/rfc/rfc4226.txt). By default, outputs a raw binary key with no encoding. The :format key can be used to specify alternate output formats. Valid formats include: :binary (default) :base32 :base64 Examples iex&gt; TwoFactorInACan.Secrets.generate_totp_secret() &lt;&lt;195, 110, 253, 36, 185, 138, 174, 16, 54, 176, 135, 67, 97, 11, 159, 63, 75, 80, 65, 6&gt;&gt; iex&gt; TwoFactorInACan.Secrets.generate_totp_secret(format: :base32) &quot;F2EJJEYSJA67QHI6DEAI2I6AGCEG7G5E&quot; iex&gt; TwoFactorInACan.Secrets.generate_totp_secret(format: :base64) &quot;xKXOSYcRVlHfnazLMlRinpb252U=&quot;"},{"ref":"TwoFactorInACan.Secrets.html#t:secret_generation_opts/0","title":"TwoFactorInACan.Secrets.secret_generation_opts/0","module":"TwoFactorInACan.Secrets","type":"type","doc":""},{"ref":"TwoFactorInACan.Totp.html","title":"TwoFactorInACan.Totp","module":"TwoFactorInACan.Totp","type":"module","doc":"Functions for working with time based one time password (TOTP) style two factory authentication as defined in RFC 4226 For details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt."},{"ref":"TwoFactorInACan.Totp.html#current_token_value/2","title":"TwoFactorInACan.Totp.current_token_value/2","module":"TwoFactorInACan.Totp","type":"function","doc":"Outputs the current TOTP token for the given secret. Expects a 160-bit binary key by default. To use secrets that are encoded the :secret_format option can be supplied. It expects one of the following values: :binary (default) :base32 :base64 This function can also accept all of the options accepted by TwoFactorInACan.Totp.time_interval/1. It simply passes them through when retrieving the time interval to pass in as the count to the TwoFactorInACan.Hotp.generate_token/3 function. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; TwoFactorInACan.Totp.current_token_value(secret) &quot;858632&quot; iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base32) iex&gt; TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base32) &quot;743622&quot; iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base64) iex&gt; TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base64) &quot;384012&quot;"},{"ref":"TwoFactorInACan.Totp.html#same_secret?/3","title":"TwoFactorInACan.Totp.same_secret?/3","module":"TwoFactorInACan.Totp","type":"function","doc":"Verifies that the provided TOTP token was generated using the provided secret. This function uses the secret to generate a token. It then compares the generated token to the supplied token. If they match, then it can be probabilistically inferred that the entity that supplied the token also knew the secret. This function allows a number of options: :acceptable_past_tokens (Default: 0) - The number of past tokens which should result in this function returning true. Setting this to 1 can be a friendly way to allow users to still verify if they have taken too long to submit their token. It should be noted that this does result in two tokens being valid instead of one, which makes a valid token easeier to guess. This value should not be set too high or security is greatly compromised. :acceptable_future_tokens (Default: 0) - The number of future tokens which should result in this function returning true. It should be noted that setting this to a nonzero value will allow more tokens to be valid and thus make a valid token easier to guess. This value should not be set too high or security is greatly compromised. :offset_seconds (Default: 0) - The number of seconds to offset the current timestamp by. If the current timestamp was 600 and the offset was 60, then the timestamp of 660 would be used to calculate the time interval. This can be useful to account for drift or difference in clock times between two entities. :interval_seconds (Default: 30) - The number of seconds that must pass before a new time_interval (and thus TOTP token) is returned by this function. This should probably never be anything but the default (30 seconds) during actual use, as nearly all apps that generate TOTP tokens assume a 30 second interval. :injected_timestamp (default: current time) - The unix timestamp to use when calculating the time interval. This should only be used during testing to ensure the same token or interval is always returned. When this option is not supplied, the current timestamp is used. :secret_format (default: :binary) - The format of the passed in secret. Can be one of :binary, :base32, or :base64. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; current_token = TwoFactorInACan.Totp.current_token_value(secret) iex&gt; TwoFactorInACan.Totp.same_secret?(secret, current_token) true"},{"ref":"TwoFactorInACan.Totp.html#time_interval/1","title":"TwoFactorInACan.Totp.time_interval/1","module":"TwoFactorInACan.Totp","type":"function","doc":"Calculates the current time interval as an integer used in the HOTP algorithm to calculate the current token value. The following options are supported: :offset_seconds (Default: 0) - The number of seconds to offset the current timestamp by. If the current timestamp was 600 and the offset was 60, then the timestamp of 660 would be used to calculate the time interval. This can be useful to account for drift or to purposefully allow the last token to be valid as well in functions that use this function. :interval_seconds (Default: 30) - The number of seconds that must pass before a new time_interval (and thus TOTP token) is returned by this function. This should probably never be anything but the default (30 seconds) during actual use, as nearly all apps that generate TOTP tokens assume a 30 second interval. :injected_timestamp (default: current time) - The unix timestamp to use when calculating the time interval. This should only be used during testing to ensure the same token or interval is always returned. When this option is not supplied, the current timestamp is used. Examples iex&gt; TwoFactorInACan.Totp.time_interval() 51802243 iex&gt; TwoFactorInACan.Totp.time_interval(offset_seconds: 30) 51802244 iex&gt; TwoFactorInACan.Totp.time_interval(interval_seconds: 60) 25901122 iex&gt; TwoFactorInACan.Totp.time_interval(injected_timestamp: 1554067403) 51802246 iex&gt; TwoFactorInACan.Totp.time_interval(injected_timestamp: 60, interval_seconds: 10) 6"},{"ref":"getting-started.html","title":"Getting Started","module":"getting-started","type":"extras","doc":"Getting Started"},{"ref":"getting-started.html#summary","title":"Getting Started - Summary","module":"getting-started","type":"extras","doc":"TwoFactorInACan is a library for adding two factor authentication (2FA) to your application. Its goal is to be the most developer friendly way to securely setup two factor authentication in the Elixir/Erlang ecosystem. That means that not only is it thoroughly documented and easy to add to your project, but it&#39;s also the easiest to understand should you need to take a look behind the curtain."},{"ref":"getting-started.html#installation","title":"Getting Started - Installation","module":"getting-started","type":"extras","doc":"This library is not currently available on hex. Once it is: The package can be installed by adding two_factor_in_a_can to your list of dependencies in mix.exs: def deps do [ {:two_factor_in_a_can, &quot;~&gt; 0.1.0&quot;} ] end"},{"ref":"roadmap.html","title":"Roadmap","module":"roadmap","type":"extras","doc":""},{"ref":"roadmap.html#roadmap","title":"Roadmap - Roadmap","module":"roadmap","type":"extras","doc":"This project is still very, very, very much under construction. That said, here&#39;s what&#39;s planned: [ ] HMAC based One Time Password 2FA (HOTP) [x] Secret Generation [x] Hotp.generate_token/3 function [ ] Hotp.verify_token/3 function [ ] Write documentation [x] Time-based One Time Password 2FA (TOTP) [x] Secret Generation [x] Totp.current_token_value/2 function [x] Totp.same_secret?/3 function [x] Write documentation [ ] Universal Second Factor (U2P) [ ] Add examples to README.md [ ] Tutorial for adding to Phoenix Maybe, but probably not 2FA via text to someone&#39;s phone (SMS) is better than no 2FA at all, but there are security users to be aware of. The main one is that it is not impossible to intercept the communication of the token when sent to the user&#39;s phone number. Ultimately, there are no cryptographic guarantees that the device used to authenticate is the same device that was originally used to setup 2FA. [ ] SMS based 2FA [ ] Thoroughly warn about not using [ ] But note why its still better than no 2FA at all [ ] Build OTP process tree for keeping state of recently sent SMS [ ] Sms.generate_and_send_token/2 function [ ] Sms.verify_token/3 function [ ] Document to pieces"}]